@@ -44,20 +44,17 @@ Lifeograph::Lifeograph( int argc, char *argv[] )
 {
 	Cipher::init();
 
-	UndoManager::m = new UndoManager;	// init()??
-	// FIXME:
+	UndoManager::m = new UndoManager;
 	Diary::d = new Diary;
 	ListData::colrec = new ListData::Colrec;
 
-	// INTERFACE INITIALIZATION
-	TextviewDiary::m_buffer = new TextbufferDiary; //EntryView::init();
-	//m_diary_view->init();
-
 	// CONFIGURATION
 	m_flag_firsttimeuser = ! settings.read();
 	set_default_size( settings.width, settings.height );
 	if( settings.position_x != POSITION_NOTSET && settings.position_y != POSITION_NOTSET )
 		move( settings.position_x, settings.position_y );
+    if( settings.state_maximized )
+        maximize();
 
 	// COMMANDLINE OPTIONS
 	for( int i = 1; i < argc; i++ )
@@ -90,48 +87,62 @@ Lifeograph::Lifeograph( int argc, char *argv[] )
 		{
 			Diary::d->m_flag_ignore_locks = true;
 		}
+        else
+        if( ! strcmp( argv[ i ], "--read-only" ) || ! strcmp( argv[ i ], "-r" ) )
+        {
+            m_flag_read_only = true;
+        }
 		else
-		if( Glib::file_test( argv[ i ], Glib::FILE_TEST_EXISTS ) )
+		if( Glib::file_test( argv[ i ], Glib::FILE_TEST_EXISTS ) &&
+		    Glib::file_test( argv[ i ], Glib::FILE_TEST_IS_DIR ) == false )
 		{
-			if( ! Glib::file_test( argv[ i ], Glib::FILE_TEST_IS_DIR ) )
-			{
 			    ViewLogin::m_path_cur = argv[ i ];
 				m_flag_open_directly = true;
-			}
 		}
+        else
+        {
+            print_info( std::string( "Unrecognized argument: " ) + argv[ i ] );
+        }
 	}
 
 	// GTKBUILDER
 	load_gui( UIDIR "/lifeograph.ui" );
 
-    //Gtk::Main::signal_run().connect( sigc::mem_fun( this, &Lifeograph::on_run ) );
-	// FIXME
-	on_run();
-}
+    // ENCHANT
+    s_enchant_broker = enchant_broker_init();
+    enchant_broker_list_dicts( s_enchant_broker, fill_lang_list_cb, NULL );
 
-Lifeograph::~Lifeograph( void )
-{
-	remove();
-	delete UndoManager::m;
-	if( m_view_login )
-	    delete m_view_login;
-	if( Diary::d ) delete Diary::d;
-}
-
-void
-Lifeograph::on_run( void )
-{
-    // purpose of this function is calling draw functions after Gtk::Main::run()
+    // DRAW UI
+    // TODO: make sure that this is execute after Gtk::Main::run()
     if( m_flag_firsttimeuser || m_flag_force_welcome )
         draw_welcomescreen();
     else
         draw_loginscreen();
 }
 
+Lifeograph::~Lifeograph( void )
+{
+    remove();
+    delete UndoManager::m;
+
+    if( m_view_login )
+        delete m_view_login;
+
+    if( Diary::d )
+        delete Diary::d;
+
+    if( s_enchant_broker )
+    {
+        enchant_broker_free( s_enchant_broker );
+        s_enchant_broker = NULL;
+	}
+}
+
 bool
 Lifeograph::on_event( GdkEvent* )
 {
-    if( ! settings.autologout || loginstatus != LOGGED_IN || ! Diary::d->is_encrypted() )
+    if( ! settings.autologout || loginstatus != LOGGED_IN || ! Diary::d->is_encrypted() ||
+        m_flag_auto_logout_frozen )
 		return false;
 	if( m_secondsremaining < LOGOUT_COUNTDOWN )
 		update_title();
@@ -159,6 +170,8 @@ Lifeograph::on_delete_event( GdkEventAny* )
 	// SAVE SETTINGS
 	get_size( settings.width, settings.height );
 	get_position( settings.position_x, settings.position_y );
+    settings.state_maximized = ( get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED );
+
 	settings.write();
 
 	return false;
@@ -185,6 +198,23 @@ Lifeograph::handle_idle()
 	return false;
 }
 
+void
+Lifeograph::freeze_auto_logout( void )
+{
+    m_flag_auto_logout_frozen = true;
+    if( m_connection_timeout.connected() )
+        m_connection_timeout.block();
+    update_title();
+}
+
+void
+Lifeograph::unfreeze_auto_logout( void )
+{
+    m_flag_auto_logout_frozen = false;
+    if( m_connection_timeout.connected() )
+        m_connection_timeout.unblock();
+}
+
 bool
 Lifeograph::write_backup( void )
 {
@@ -285,12 +315,15 @@ Lifeograph::finish_editing( bool opt_save )
 	// CLEARING
 	// TODO: m_loginstatus = LOGGING_OUT_IN_PROGRESS;
 
+    m_actions_read->set_sensitive( false );
+
 	m_signal_logout.emit();	// only for DialogEvent
 
 	m_internaloperation++;
+	panel_main->handle_logout();
 	panel_diary->handle_logout();
-	view_entry->m_textviewdiary->m_buffer->handle_logout();
-	panel_main->clear_history();
+	view_entry->get_buffer()->handle_logout();
+	panel_extra->handle_logout();
 
 	if( loginstatus == LOGGED_IN )
 		loginstatus = LOGGED_OUT;
@@ -327,7 +360,7 @@ Lifeograph::confirm_dismiss_element( void )
 }
 
 Gtk::TreeRow
-Lifeograph::get_element_row( DiaryElement *element )
+Lifeograph::get_element_row( const DiaryElement *element )
 {
 	switch( element->get_type() )
 	{
@@ -336,18 +369,14 @@ Lifeograph::get_element_row( DiaryElement *element )
 			return( panel_diary->get_row( element->m_list_data->treepath ) );
 
 		case DiaryElement::ET_TAG:
-			return( panel_tag->get_row( element->m_list_data->treepath ) );
+		case DiaryElement::ET_TAG_CTG:
+		case DiaryElement::ET_THEME:
+			return( panel_extra->get_row( element->m_list_data->treepath ) );
 		default:
 			return( Gtk::TreeRow() );
 	}
 }
 
-void
-Lifeograph::update_theme_list( void ) // TODO: to be removed along with theme list
-{
-	m_widget_panel_exp->populate_themes();
-}
-
 void
 Lifeograph::update_title( void )
 {
@@ -409,35 +438,6 @@ Lifeograph::draw_loginscreen( void )
     // only to be called on start up
     assert( m_view_login == NULL );
 
-    // DETECT STOCK DIARIES
-    Gio::init();
-
-    try
-    {
-        Glib::RefPtr< Gio::File > directory = Gio::File::create_for_path( DIARYDIR );
-        if( directory )
-        {
-            Glib::RefPtr< Gio::FileEnumerator > enumerator = directory->enumerate_children();
-            if( enumerator )
-            {
-                Glib::RefPtr< Gio::FileInfo > file_info = enumerator->next_file();
-                while( file_info )
-                {
-                    std::string path( DIARYDIR );
-                    path += "/";
-                    path += file_info->get_name();
-                    //std::cout << "file: " << file_info->get_name() << std::endl;
-                    stock_diaries.insert( path );
-                    file_info = enumerator->next_file();
-                }
-            }
-        }
-    }
-    catch( const Glib::Exception& ex )
-    {
-        LIFEO::Error( ex.what() );
-    }
-
     try
     {
         m_view_login = new ViewLogin;
@@ -450,68 +450,77 @@ Lifeograph::draw_loginscreen( void )
 }
 
 void
-Lifeograph::draw_editscreen( void )
+Lifeograph::login( void )
 {
-	remove();
+    remove();
 
-	if( m_hpaned_main == NULL )
-	{
+    if( m_hpaned_main == NULL )
+    {
         Gtk::MenuItem               *menuitem_preferences;
         Gtk::MenuItem               *menuitem_about;
 
-		try
-		{
-			Lifeobase::builder->get_widget( "hpaned_main", m_hpaned_main );
-			// TOOLBAR AND ITS ITEMS
-			Lifeobase::builder->get_widget( "button_logout", m_button_logout );
-
+        try
+        {
+            // GENERAL TOOLS
             Lifeobase::builder->get_widget( "menuitem_preferences", menuitem_preferences );
             Lifeobase::builder->get_widget( "menuitem_about", menuitem_about );
 
-            // TIME BASED NAVIGATION PANE
+            // PANES
+            Lifeobase::builder->get_widget( "hpaned_main", m_hpaned_main );
             Lifeobase::builder->get_widget( "hpaned_second", m_hpaned_entry );
 
-			panel_main = new PanelMain;
-			panel_diary = new PanelDiary;
-			panel_tag = new TagPanel;
-			m_widget_panel_exp = new WidgetPanelExp;
-			m_diary_view = new DiaryView;
-			m_theme_view = new ThemeView;
-			m_category_tags_view = new CategoryTagsView;
-			m_chapter_view = new ChapterView;
-			view_entry = new EntryView;
-			view_tag = new TagView;
-		}
-		catch( std::exception &ex )
-		{
-			throw ex;
-		}
+            // TAG PANEL
+            Lifeobase::builder->get_widget_derived( "treeview_all_tags", panel_extra );
+
+            // ICONS
+            Glib::RefPtr< Gtk::IconTheme > theme = Gtk::IconTheme::get_default();
+            icons->diary_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/diary-16.png" );
+            icons->diary_32 = Gdk::Pixbuf::create_from_file( ICONDIR "/diary-32.png" );
+            icons->entry_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/entry-16.png" );
+            icons->entry_32 = Gdk::Pixbuf::create_from_file( ICONDIR "/entry-32.png" );
+            icons->entry_favorite_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/favorite-16.png" );
+            icons->entry_favorite_32 = Gdk::Pixbuf::create_from_file( ICONDIR "/favorite-32.png" );
+            icons->tag_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/tag-16.png" );
+            icons->tag_32 = Gdk::Pixbuf::create_from_file( ICONDIR "/tag-32.png" );
+            icons->tag_category_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/category_tags-16.png" );
+            icons->tag_category_32 = Gdk::Pixbuf::create_from_file( ICONDIR "/category_tags-32.png" );
+            icons->chapter_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/chapter-16.png" );
+            icons->chapter_32 = Gdk::Pixbuf::create_from_file( ICONDIR "/chapter-32.png" );
+            icons->theme_16 = theme->load_icon( "gtk-select-color", 16 );
+            icons->theme_default_16 = theme->load_icon( "gtk-apply", 16 );
+            icons->theme_32 = theme->load_icon( "gtk-select-color", 32 );
+            icons->month_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/month-16.png" );
+            icons->month_32 = Gdk::Pixbuf::create_from_file( ICONDIR "/month-32.png" );
+            icons->filter_16 = Gdk::Pixbuf::create_from_file( ICONDIR "/filter-16.png" );
+
+            // PANELS & VIEWS
+            panel_main = new PanelMain;
+            panel_diary = new PanelDiary;
+            //panel_extra = new PanelExtra;
+            m_diary_view = new DiaryView;
+            m_theme_view = new ThemeView;
+            m_category_tags_view = new CategoryTagsView;
+            m_chapter_view = new ChapterView;
+            view_entry = new EntryView;
+            view_tag = new TagView;
+        }
+        catch( std::exception &ex )
+        {
+            throw ex;
+        }
 
         // GEOMETRY
         m_hpaned_main->set_position( settings.position_pane );
         m_hpaned_entry->set_position( settings.position_pane_tags );
 
-		m_button_logout->add_accelerator(
-				"clicked",
-				get_accel_group(),
-				GDK_KEY_Escape,
-				Gdk::CONTROL_MASK,
-				Gtk::ACCEL_VISIBLE );
-
 		// ACTIONS
-        create_action( m_action_undo,
-                       "Undo",
-                       Gtk::Stock::UNDO,
-                       _( "Undo" ),
-                       "",     // this tooltip will be dynamic
+        create_action( true, m_action_undo,"Undo", Gtk::Stock::UNDO,
+                       _( "Undo" ), "",     // this tooltip will be dynamic
                        Gtk::AccelKey( GDK_KEY_z, Gdk::CONTROL_MASK ),
                        sigc::mem_fun( *this, &Lifeograph::handle_undo ) );
 
-        create_action( m_action_redo,
-                       "Redo",
-                       Gtk::Stock::REDO,
-                       _( "Redo" ),
-                       "",     // this tooltip will be dynamic
+        create_action( true, m_action_redo, "Redo", Gtk::Stock::REDO,
+                       _( "Redo" ), "",     // this tooltip will be dynamic
                        Gtk::AccelKey( GDK_KEY_z, Gdk::CONTROL_MASK | Gdk::SHIFT_MASK ),
                        sigc::mem_fun( *this, &Lifeograph::handle_redo ) );
 
@@ -520,22 +529,16 @@ Lifeograph::draw_editscreen( void )
 				sigc::mem_fun( this, &Lifeograph::show_about ) );
 		menuitem_preferences->signal_activate().connect(
 				sigc::ptr_fun( &DialogPreferences::create ) );
-
-		m_button_logout->signal_clicked().connect(
-				sigc::bind( sigc::mem_fun( *this, &Lifeograph::logout ), true ) );
-	}
-	else
-	{
-		m_internaloperation++;
-		panel_diary->m_entry_filter->set_text( "" );
-		panel_diary->m_entry_replace->set_text( "" );
-		m_internaloperation--;
 	}
 
 	m_connection_backup = Glib::signal_timeout().connect_seconds(
 			sigc::mem_fun( this, &Lifeograph::write_backup ),
 			BACKUP_INTERVAL );
 
+    m_actions_read->set_sensitive( true );
+    m_actions_edit->set_visible( Diary::d->is_read_only() == false );
+    m_actions_edit->set_sensitive( Diary::d->is_read_only() == false );
+
 	add( *m_hpaned_main );
 
 	// LOGIN
@@ -545,10 +548,11 @@ Lifeograph::draw_editscreen( void )
 	m_diary_view->handle_login();
 	view_entry->handle_login();
 	panel_diary->handle_login();
-	panel_tag->populate_main();
-	m_widget_panel_exp->populate_themes();
+	panel_extra->populate();
 
 	m_internaloperation--;
+
 	loginstatus = LOGGED_IN;
+    m_flag_auto_logout_frozen = false;
 	update_title();
 }
