@@ -42,19 +42,19 @@ PanelDiary::PanelDiary( void )
     Gtk::Button *button_today;
 	try
 	{
-        Menubutton *mb_tools;
-
+        Lifeobase::builder->get_widget( "button_logout", m_button_logout );
         Lifeobase::builder->get_widget( "button_today", button_today );
 
+        Menubutton *mb_tools;
         Menubutton::s_builder_name = "menubutton_tools";
         Lifeobase::builder->get_widget_derived( Menubutton::s_builder_name, mb_tools );
         Lifeobase::builder->get_widget( "menuitem_sort_by_date", m_menuitem_sort_by_date );
         Lifeobase::builder->get_widget( "menuitem_sort_by_size", m_menuitem_sort_by_size );
-
-        Lifeobase::builder->get_widget( "sprt_tb_1", m_sprt_tb1 );
+        Lifeobase::builder->get_widget( "menuitem_sort_by_change", m_menuitem_sort_by_change );
 
         Lifeobase::builder->get_widget( "togglebtn_filter_favorites",
                                         m_togglebtn_filter_favorites );
+        Lifeobase::builder->get_widget( "togglebtn_filter_trash", m_togglebtn_filter_trash );
         Lifeobase::builder->get_widget( "button_remove_filters", m_button_remove_filters );
 
 		Lifeobase::builder->get_widget( "calendar_main", m_calendar );
@@ -80,6 +80,9 @@ PanelDiary::PanelDiary( void )
 	m_entry_replace->set_placeholder_text( _( "Replace" ) );
 
 	// SIGNALS
+    m_button_logout->signal_clicked().connect(
+            sigc::bind( sigc::mem_fun( Lifeobase::base, &Lifeobase::logout ), true ) );
+
     m_menuitem_sort_by_date->signal_activate().connect(
             sigc::bind(
                     sigc::mem_fun( m_treeview_entries,
@@ -90,9 +93,18 @@ PanelDiary::PanelDiary( void )
                     sigc::mem_fun( m_treeview_entries,
                                    &WidgetEntryList::handle_sorting_criteria_changed ),
                     SC_SIZE ) );
+    m_menuitem_sort_by_change->signal_activate().connect(
+            sigc::bind(
+                    sigc::mem_fun( m_treeview_entries,
+                                   &WidgetEntryList::handle_sorting_criteria_changed ),
+                    SC_CHANGE ) );
+
     m_togglebtn_filter_favorites->signal_toggled().connect(
             sigc::mem_fun( this, &PanelDiary::handle_filter_favorites_toggled ) );
 
+    m_togglebtn_filter_trash->signal_toggled().connect(
+            sigc::mem_fun( this, &PanelDiary::handle_filter_trash_toggled ) );
+
     m_button_remove_filters->signal_clicked().connect(
             sigc::mem_fun( this, &PanelDiary::remove_filters ) );
 
@@ -129,39 +141,32 @@ PanelDiary::PanelDiary( void )
 
     // ACTIONS
     Lifeobase::create_action(
-            m_action_today,
-            "Today",
-            "x-office-calendar",
-            _( "Today" ),
+            true, m_action_today, "Today", "x-office-calendar", _( "Today" ),
             _( "Go to today (creates a new entry if there isn't any or when already at today)" ),
             Gtk::AccelKey( GDK_KEY_d, Gdk::CONTROL_MASK ),
             sigc::mem_fun( this, &PanelDiary::show_today ),
             button_today );
 
     Lifeobase::create_action(
-            m_action_focusfilter,
-            "FocusFilter",
-            Gtk::Stock::FIND,
-            "", "",     // no grachical interface yet!
+            false, m_action_focusfilter, "FocusFilter",
+            Gtk::Stock::FIND, "", "",     // no grachical interface yet!
             Gtk::AccelKey( GDK_KEY_f, Gdk::CONTROL_MASK ),
             sigc::mem_fun( m_entry_filter, &Gtk::Widget::grab_focus ) );
 
     Lifeobase::create_action(
-            m_action_previous_list, "ListPrevious", Gtk::Stock::GO_UP, _( "Previous" ),
+            false, m_action_previous_list, "ListPrevious", Gtk::Stock::GO_UP, _( "Previous" ),
             _( "Go to the previous entry in the list" ),
             Gtk::AccelKey( GDK_KEY_Page_Up, Gdk::CONTROL_MASK ),
             sigc::bind( sigc::mem_fun( m_treeview_entries, &WidgetEntryList::go_up ), false ) );
 
     Lifeobase::create_action(
-            m_action_next_list, "ListNext", Gtk::Stock::GO_DOWN, _( "Next" ),
+            false, m_action_next_list, "ListNext", Gtk::Stock::GO_DOWN, _( "Next" ),
             _( "Go to the next entry in the list" ),
             Gtk::AccelKey( GDK_KEY_Page_Down, Gdk::CONTROL_MASK ),
             sigc::bind( sigc::mem_fun( m_treeview_entries, &WidgetEntryList::go_down ), false ) );
 
     Lifeobase::create_action(
-            m_action_previous_match,
-            "MatchPrevious",
-            Gtk::Stock::GO_BACK,
+            false, m_action_previous_match, "MatchPrevious", Gtk::Stock::GO_BACK,
 // TRANSLATORS: label and tooltip of the go-to-previous-match-of-filtered-text action
             _( "Previous Match" ),
             _( "Go to previous match" ),
@@ -170,9 +175,7 @@ PanelDiary::PanelDiary( void )
             m_button_match_prev );
 
     Lifeobase::create_action(
-            m_action_next_match,
-            "MatchNext",
-            Gtk::Stock::GO_FORWARD,
+            false, m_action_next_match, "MatchNext", Gtk::Stock::GO_FORWARD,
 // TRANSLATORS: label and tooltip of the go-to-next-match-of-filtered-text action
             _( "Next Match" ),
             _( "Go to next match" ),
@@ -181,21 +184,24 @@ PanelDiary::PanelDiary( void )
             m_button_match_next );
 
     Lifeobase::create_action(
-            m_action_jump2current,
-            "Jump",
-            Gtk::Stock::JUMP_TO,
+            false, m_action_jump2current, "Jump", Gtk::Stock::JUMP_TO,
             "", "",     // no grachical interface yet!
             Gtk::AccelKey( GDK_KEY_j, Gdk::CONTROL_MASK ),
             sigc::mem_fun( this, &PanelDiary::present_current_row ) );
 
     Lifeobase::create_action(
-            m_action_show_prev_session_elem,
-            "PrevSessElem",
-            Gtk::Stock::JUMP_TO,
+            false, m_action_show_prev_session_elem, "PrevSessElem", Gtk::Stock::JUMP_TO,
             "", "",     // no grachical interface yet!
             Gtk::AccelKey( GDK_KEY_l, Gdk::CONTROL_MASK ),
             sigc::mem_fun( this, &PanelDiary::show_prev_session_elem ) );
 
+    m_button_logout->add_accelerator(
+            "clicked",
+            Lifeobase::base->get_accel_group(),
+            GDK_KEY_Escape,
+            Gdk::CONTROL_MASK,
+            Gtk::ACCEL_VISIBLE );
+
     // becoming proxy widgets makes them visible:
     m_button_match_prev->set_visible( false );
     m_button_match_next->set_visible( false );
@@ -275,11 +281,11 @@ PanelDiary::update_entry_list( void )
 	PRINT_DEBUG( "update_entry_list()" );
 	Lifeobase::m_internaloperation++;
 
+	Lifeobase::view_entry->sync();
+
 	l_itr_entry = Diary::d->m_entries.begin();
 	l_entry = ( l_itr_entry != Diary::d->m_entries.end() ? l_itr_entry->second : NULL );
 
-	Lifeobase::view_entry->sync();
-
 	m_treeview_entries->m_treestore_entries->clear();
 	Lifeobase::m_entrycount = 0;
 
@@ -296,7 +302,8 @@ PanelDiary::update_entry_list( void )
 	// CHAPTERS
 	//CategoryChapters *chapters = Diary::d->get_current_chapter_ctg();
 
-	if( Diary::d->get_sorting_criteria() == SC_DATE )
+    if( Diary::d->get_sorting_criteria() == SC_DATE &&
+        !( Diary::d->get_filtering_status() & Diary::FS_FILTER_TRASH ) )
 	{
 		add_chapter_category_to_list( Diary::d->get_topics() );
 		add_chapter_category_to_list( Diary::d->get_current_chapter_ctg() );
@@ -340,6 +347,8 @@ PanelDiary::update_entry_list( void )
 		elem = Diary::d->get_startup_elem();
 		if( elem == NULL )	// if last item was empty and dismissed at logout
 			elem = Diary::d;
+        else if( elem->get_filtered_out() ) // if last item is filtered out (trashed)
+            elem = Diary::d->get_entry_first();
 	}
 
 	if( flag_show_elem )
@@ -413,10 +422,19 @@ PanelDiary::update_button_remove_filters( void )
 void
 PanelDiary::handle_login( void )
 {
-	if( Diary::d->get_sorting_criteria() == SC_DATE )
-		m_menuitem_sort_by_date->set_active();
-	else
-		m_menuitem_sort_by_size->set_active();
+    switch( Diary::d->get_sorting_criteria() )
+    {
+        default: // future-proofing
+        case SC_DATE: // no break
+            m_menuitem_sort_by_date->set_active();
+            break;
+        case SC_SIZE:
+            m_menuitem_sort_by_size->set_active();
+            break;
+        case SC_CHANGE:
+            m_menuitem_sort_by_change->set_active();
+            break;
+    }
 
 	// not called automatically because of m_internaloperation:
 	m_treeview_entries->set_sorting_criteria();
@@ -424,22 +442,25 @@ PanelDiary::handle_login( void )
 
     m_button_remove_filters->set_visible( false );
 
-	m_action_today->set_visible( ! Diary::d->is_read_only() );
 	m_button_add->set_visible( ! Diary::d->is_read_only() );
-	m_sprt_tb1->set_visible( ! Diary::d->is_read_only() );
 }
 
 void
 PanelDiary::handle_logout( void )
 {
+    Lifeobase::m_internaloperation++;
+
 	m_entry_filter->set_text( "" );
 	m_entry_replace->set_text( "" );
 	m_togglebtn_filter_favorites->set_active( false );
-	m_treeview_entries->m_treestore_entries->clear();
+    m_togglebtn_filter_trash->set_active( false );
+	m_treeview_entries->clear();
 
     m_button_match_prev->hide();
     m_button_match_next->hide();
     m_toolbar_replace->hide();
+
+    Lifeobase::m_internaloperation--;
 }
 
 void
@@ -455,7 +476,7 @@ PanelDiary::show( const DiaryElement *element )
         select_date_in_calendar( element->get_date() );
         m_menuitem_add_entry->set_sensitive( true );
         m_menuitem_add_chapter->set_sensitive(
-                Diary::d->get_current_chapter_ctg()->get_chapter( element->get_date() )
+                Diary::d->get_current_chapter_ctg()->get_chapter( element->get_date().m_date )
                 == NULL );
     }
 	else
@@ -473,7 +494,12 @@ PanelDiary::show_today( void )
     bool flag_add = false;
 
     if( entry_today == NULL )   // add new entry if no entry exists on selected date
-        flag_add = true;
+    {
+        if( Diary::d->is_read_only() )
+            return;
+        else
+            flag_add = true;
+    }
     else                        // or current entry is already at today
     if( entry_today->get_date().get_pure() ==
                 Lifeobase::panel_main->get_cur_elem()->get_date().get_pure() )
@@ -515,20 +541,20 @@ PanelDiary::handle_entry_filter_changed( void )
 	if( Lifeobase::m_internaloperation ) return;
 	// current entry must be closed here or else it loses...
 	// ...changes since it was selected:
-	Lifeobase::view_entry->sync();
+	//Lifeobase::view_entry->sync(); update_entry_list() does the syncing
 
 	const std::string filterstring( m_entry_filter->get_text().lowercase() );
 
 	Diary::d->set_filter_text( filterstring );
-	update_entry_list();
+	update_entry_list();    // syncs current entry
 
 	if( filterstring.size() > 0 )
 	{
 		if( Lifeobase::m_entrycount > 0 )
 		{
-			TextviewDiary::m_buffer->set_search_str( filterstring );
-			if( Lifeobase::panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY )
-				TextviewDiary::m_buffer->select_searchstr_next();
+            Lifeobase::view_entry->get_buffer()->set_search_str( filterstring );
+            if( Lifeobase::panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY )
+                Lifeobase::view_entry->get_buffer()->select_searchstr_next();
 		}
 
 		m_entry_filter->set_tooltip_markup(
@@ -538,7 +564,7 @@ PanelDiary::handle_entry_filter_changed( void )
 	}
 	else
 	{
-		TextviewDiary::m_buffer->set_search_str( "" );
+	    Lifeobase::view_entry->get_buffer()->set_search_str( "" );
 		m_entry_filter->set_tooltip_text( _( "Enter text to be filtered" ) );
 	}
 
@@ -568,6 +594,15 @@ PanelDiary::handle_filter_favorites_toggled( void )
 	update_entry_list();					// apply the filter
 }
 
+void
+PanelDiary::handle_filter_trash_toggled( void )
+{
+    if( Lifeobase::m_internaloperation ) return;
+
+    Diary::d->toggle_filter_trash();    // set filter
+    update_entry_list();                // apply the filter
+}
+
 void
 PanelDiary::remove_filters( void )
 {
@@ -577,8 +612,10 @@ PanelDiary::remove_filters( void )
         if( tag != NULL )
         {
             Diary::d->set_filter_tag( NULL );
-            Lifeobase::panel_tag->update_tag_icon( tag );
             Lifeobase::view_tag->update_button_filter_tag();
+            // UPDATE ICON
+            Gtk::TreeRow row = * Lifeobase::base->get_element_row( tag );
+            row[ PanelExtra::colrec->icon ] = tag->get_icon();
         }
         m_button_remove_filters->set_visible( false );
 
@@ -599,7 +636,7 @@ PanelDiary::handle_calendar_dayselected( void )
 
     m_menuitem_add_entry->set_sensitive( day > 0 );
     m_menuitem_add_chapter->set_sensitive( day > 0 &&
-            Diary::d->get_current_chapter_ctg()->get_chapter( date ) == NULL );
+            Diary::d->get_current_chapter_ctg()->get_chapter( date.m_date ) == NULL );
 
 	Entry *entry;
 	if( date.m_date == ( Lifeobase::panel_main->get_cur_elem()->get_date().get_pure() ) )
@@ -665,7 +702,7 @@ PanelDiary::handle_add_chapter( void )
     m_calendar->get_date( year, month, day );
     Date date( year, month + 1, day );
 
-    if( day > 0 && Diary::d->get_current_chapter_ctg()->get_chapter( date ) == NULL )
+    if( day > 0 && Diary::d->get_current_chapter_ctg()->get_chapter( date.m_date ) == NULL )
     {
         Chapter *c( Diary::d->get_current_chapter_ctg()->create_chapter(
                 _( "Untitled chapter" ), date ) );
@@ -698,25 +735,26 @@ PanelDiary::select_date_in_calendar( const Date &date )
 void
 PanelDiary::go_prev_match( void )
 {
-	if( Lifeobase::m_entrycount < 1 )
-		return;
+    if( Lifeobase::m_entrycount < 1 )
+        return;
 
-	// TODO: move to Entry
-	if( Lifeobase::panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY )
-		if( TextviewDiary::m_buffer->select_searchstr_previous() )
-			return;
+    // TODO: move to Entry
+    if( Lifeobase::panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY )
+        if( Lifeobase::view_entry->get_buffer()->select_searchstr_previous() )
+            return;
 
-	int i = 0;
-	do
-	{
+    int i = 0;
+    do
+    {
         m_treeview_entries->go_up( true );
-		// put cursor to the end:
-		TextviewDiary::m_buffer->select_range( TextviewDiary::m_buffer->end(),
-											   TextviewDiary::m_buffer->end() );
-		i++;
-	}
-	while( ! TextviewDiary::m_buffer->select_searchstr_previous() &&
-		   i < Lifeobase::m_entrycount );
+        // put cursor to the end:
+        Lifeobase::view_entry->get_buffer()->select_range(
+                Lifeobase::view_entry->get_buffer()->end(),
+                Lifeobase::view_entry->get_buffer()->end() );
+        i++;
+    }
+    while( ! Lifeobase::view_entry->get_buffer()->select_searchstr_previous() &&
+           i < Lifeobase::m_entrycount );
 }
 
 void
@@ -727,7 +765,7 @@ PanelDiary::go_next_match( void )
 
 	// TODO: move to Entry
 	if( Lifeobase::panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY )
-		if( TextviewDiary::m_buffer->select_searchstr_next() )
+		if( Lifeobase::view_entry->get_buffer()->select_searchstr_next() )
 			return;
 
 	int i = 0;
@@ -736,19 +774,19 @@ PanelDiary::go_next_match( void )
         m_treeview_entries->go_down( true );
 		i++;
 	}
-	while( ! TextviewDiary::m_buffer->select_searchstr_next() &&
+	while( ! Lifeobase::view_entry->get_buffer()->select_searchstr_next() &&
 		   i < Lifeobase::m_entrycount );
 }
 
 void
 PanelDiary::replace_match( void )
 {
-	if( ! TextviewDiary::m_buffer->get_has_selection() )
+	if( ! Lifeobase::view_entry->get_buffer()->get_has_selection() )
 		go_next_match();
-	if( TextviewDiary::m_buffer->get_has_selection() )
+	if( Lifeobase::view_entry->get_buffer()->get_has_selection() )
 	{
-		TextviewDiary::m_buffer->erase_selection();
-		TextviewDiary::m_buffer->insert_at_cursor( m_entry_replace->get_text() );
+		Lifeobase::view_entry->get_buffer()->erase_selection();
+		Lifeobase::view_entry->get_buffer()->insert_at_cursor( m_entry_replace->get_text() );
 		go_next_match();
 	}
 }
@@ -763,7 +801,7 @@ PanelDiary::replace_all_matches( void )
 	if( Lifeobase::panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY )
 	{
 		Entry *entry = dynamic_cast< Entry* >( Lifeobase::panel_main->get_cur_elem() );
-		TextviewDiary::m_buffer->set_richtext( entry );
+		Lifeobase::view_entry->get_buffer()->set_richtext( entry );
 	}
 }
 
